-----------------------------------------------------------
---------------------- By Mystical ------------------------
-----------------------------------------------------------
cameraz	= -1200
love.graphics.setDefaultFilter("nearest", "nearest")

-- map ---------------------------------------------------
	heightMap = love.graphics.newImage("heightMap.png")
	colourMap = love.graphics.newImage("colourMap.png")
----------------------------------------------------------

-- player ------------------------------------------------
player = {
			x = 0,
			y = 0.5,
			
			a = 0,
			h = 0.2,
			p = -0.5,
		 }
----------------------------------------------------------

function love.load()
raycastShader = love.graphics.newShader(
	[[
		extern float cameraz; // depth
		
		extern float playerX;
		extern float playerY;
		extern float playerHeight; // view height
		extern float angle; // view angle
		extern float viewPlane;

		extern Image heightMap;
		extern Image colourMap;
		
		vec2 rayPos = vec2(playerX, playerY);
		
		vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 screen_coords)
		{
			float camerax = screen_coords.x - love_ScreenSize.x/2;
			float screenNormal = screen_coords.y/love_ScreenSize.y;
			float screenNormalCenter = min(screen_coords.y/love_ScreenSize.y-viewPlane-playerHeight+1.5, 1);
			
			float rotx = cos(angle) * camerax + sin(angle) * cameraz;
			float roty = -sin(angle) * camerax + cos(angle) * cameraz;

			vec2 rayDir = vec2(rotx/cameraz, roty/cameraz);

			// cast the rays
			for (float t = 0.04; t < 0.5*(screenNormalCenter); t += 0.0009)
			{
				float length = (pow(t*5, 2)/5);
				vec2 ray = rayPos + rayDir*length;
				
				float heightValue  = Texel(heightMap, mod(ray, 1)).r;
				vec3 colourValue   = Texel(colourMap, mod(ray, 1)).rgb;
				
				if((screenNormal+viewPlane)*length + playerHeight < heightValue)
				{
					return vec4(colourValue, 1.0/pow(t+0.6, 32));
				}
			}
			discard;
		}
	]])

	love.window.setMode(640, 400)
	w, h = love.graphics.getDimensions()
	
	raycastShader:send('cameraz', cameraz)

	raycastShader:send('heightMap', heightMap)
	raycastShader:send('colourMap', colourMap)
end

function love.draw()
	love.graphics.setColor(40, 120, 235)
	love.graphics.rectangle("fill", 0, 0, w, h)
	love.graphics.setColor(255,255,255)
	
    love.graphics.setShader(raycastShader)
	love.graphics.rectangle("fill", 0, 0, w, h)
    love.graphics.setShader()
	
	love.graphics.print("Player X: " .. player.x .. "\n" .. "Player Y: " .. player.y)
end

function love.update(dt)
	love.window.setTitle("2.5D Voxelcaster | FPS: " .. love.timer.getFPS())
	
	raycastShader:send('playerX'		, player.x)
	raycastShader:send('playerY'		, player.y)
	raycastShader:send('angle'			, player.a)
	raycastShader:send('playerHeight'	, player.h)
	raycastShader:send('viewPlane'		, player.p)

	local mod = dt/10
	
	if love.keyboard.isDown("w") then 
		player.x = player.x + mod * math.cos(player.a - math.pi/2)
		player.y = player.y - mod * math.sin(player.a - math.pi/2) 
	end
	
	if love.keyboard.isDown("s") then
		player.x = player.x - mod * math.cos(player.a - math.pi/2)
		player.y = player.y + mod * math.sin(player.a - math.pi/2)
	end
	
	if love.keyboard.isDown("a") then player.a = player.a + mod*10 end
	if love.keyboard.isDown("d") then player.a = player.a - mod*10 end
	
	if love.keyboard.isDown("r") then player.h = player.h + mod end
	if love.keyboard.isDown("f") then player.h = player.h - mod end
	
	if love.keyboard.isDown("t") then player.p = player.p + mod*10 end
	if love.keyboard.isDown("g") then player.p = player.p - mod*10 end
end